bl_info = {
	"name": "Lucas' Pure3D Editor Skin XML Format",
	"author": "Lucas Cardellini",
	"version": (1,0,0),
	"blender": (2,77,0),
	"location": "File > Import-Export",
	"description": "Import and export skin XML files for use with Lucas' Pure3D Editor.",
	"wiki_url": "http://sharmods.donutteam.com/",
	"tracker_url": "http://donutteam.com/forum",
	"support": "COMMUNITY",
	"category": "Import-Export"}

if "bpy" in locals():
	import importlib
	if "import_lucas_pure3d_editor_skin_xml" in locals():
		importlib.reload(import_lucas_pure3d_editor_skin_xml)
	if "export_lucas_pure3d_editor_skin_xml" in locals():
		importlib.reload(export_lucas_pure3d_editor_skin_xml)

import bpy
from bpy.props import (
		StringProperty,
		BoolProperty,
		)
from bpy_extras.io_utils import (
		ImportHelper,
		ExportHelper,
		)

class ImportLucasPure3DEditorSkinXML(bpy.types.Operator, ImportHelper):
	bl_idname = "import_scene.lucas_pure3d_editor_skin_xml"
	bl_label = "Import Lucas' Pure3D Editor Skin XML"
	bl_options = {"UNDO"}
	
	filename_ext = ".xml"
	filter_glob = StringProperty(default="*.xml",options={"HIDDEN"})
	
	def execute(Self, Context):
		from . import import_lucas_pure3d_editor_skin_xml
		
		return import_lucas_pure3d_editor_skin_xml.Load(Self,Context,**Self.as_keywords(ignore=("filter_glob",)))

class ExportLucasPure3DEditorSkinXML(bpy.types.Operator, ExportHelper):
	bl_idname = "export_scene.lucas_pure3d_editor_skin_xml"
	bl_label = "Export Lucas' Pure3D Editor Skin XML"
	bl_options = {"PRESET"}

	filename_ext = ".xml"
	filter_glob = StringProperty(default="*.xml",options={"HIDDEN"})
	
	use_selection = BoolProperty(
			name="Selection Only",
			description="Export selected objects only",
			default=False,
			)

	use_mesh_modifiers = BoolProperty(
			name="Apply Modifiers",
			description="Apply modifiers (preview resolution)",
			default=True,
			)
	
	use_visible = BoolProperty(
			name="Visible Only",
			description="Export visible objects only",
			default=True,
			)

	use_mesh_transformation = BoolProperty(
			name="Apply Transformations",
			description="Apply transformations",
			default=True,
			)

	def execute(Self, Context):
		from . import export_lucas_pure3d_editor_skin_xml

		return export_lucas_pure3d_editor_skin_xml.Save(Self,Context,**Self.as_keywords(ignore=("check_existing","filter_glob",)))

def menu_func_import(Self, Context):
	Self.layout.operator(ImportLucasPure3DEditorSkinXML.bl_idname,text="Lucas' Pure3D Editor Skin (.xml)")

def menu_func_export(self, context):
	self.layout.operator(ExportLucasPure3DEditorSkinXML.bl_idname,text="Lucas' Pure3D Editor Skin (.xml)")

def register():
	bpy.utils.register_module(__name__)
	
	bpy.types.INFO_MT_file_import.append(menu_func_import)
	bpy.types.INFO_MT_file_export.append(menu_func_export)

def unregister():
	bpy.utils.unregister_module(__name__)
	
	bpy.types.INFO_MT_file_import.remove(menu_func_import)
	bpy.types.INFO_MT_file_export.remove(menu_func_export)

if __name__ == "__main__":
	register()