import bpy
import xml.etree.cElementTree
import bmesh
import mathutils

class Vertex:
	def __init__(Self, VertexIndex, Normal, UV):
		Self.VertexIndex = VertexIndex
		Self.Normal = Normal
		Self.UV = UV

class Shader:
	def __init__(Self, Element):
		Self.Element = Element
		Self.Vertices = []

	def GetVertexIndex(Self, VertexIndex, Normal, UV):
		I = 0
		VertexCount = len(Self.Vertices)
		while I < VertexCount:
			Vertex2 = Self.Vertices[I]
			if Vertex2.VertexIndex == VertexIndex and Vertex2.Normal == Normal and Vertex2.UV == UV:
				return I
			I += 1
		return None

def Save(Operator, Context, filepath, use_selection, use_mesh_modifiers, use_visible, use_mesh_transformation):
	if bpy.ops.object.mode_set.poll():
		bpy.ops.object.mode_set(mode="OBJECT")
	RootElement = xml.etree.cElementTree.Element("Skin")
	Scene = bpy.context.scene
	Joints = set()
	Shaders2 = set()
	if use_selection:
		Objects = Context.selected_objects
	else:
		Objects = bpy.data.objects
	for Object in Objects:
		if Object.type == "MESH":
			if not use_visible or not Object.hide:
				VertexCount = 0
				PrimitiveCount = 0
				LoopVertexIndices = {}
				Shaders = {}
				Mesh = Object.to_mesh(Scene,use_mesh_modifiers,"PREVIEW",calc_tessface=False)
				if use_mesh_transformation:
					Mesh.transform(Object.matrix_world)
				BMesh = bmesh.new()
				BMesh.from_mesh(Mesh)
				bmesh.ops.triangulate(BMesh,faces=BMesh.faces)
				BMesh.to_mesh(Mesh)
				BMesh.free()
				Mesh.calc_normals_split()
				for VertexGroup in Object.vertex_groups:
					if not VertexGroup.name in Joints:
						JointElement = xml.etree.cElementTree.SubElement(RootElement,"Joint")
						JointElement.set("Name",VertexGroup.name)
						Joints.add(VertexGroup.name)
				for Material in Mesh.materials:
					if not Material.name in Shaders2:
						ShaderElement = xml.etree.cElementTree.SubElement(RootElement,"Shader")
						ShaderElement.set("Name",Material.name)
						for TextureSlot in Material.texture_slots:
							if not TextureSlot is None:
								if TextureSlot.use_map_color_diffuse:
									if TextureSlot.texture.type == "IMAGE":
										ShaderElement.set("TextureName",TextureSlot.texture.name)
										break
						Shaders2.add(Material.name)
				MeshElement = xml.etree.cElementTree.SubElement(RootElement,"Mesh")
				MeshElement.set("Name",Object.name)
				UVLayer = Mesh.uv_layers.active
				PolygonCount = len(Mesh.polygons)
				PrimitiveIndex = 0
				while PrimitiveIndex < PolygonCount:
					Polygon = Mesh.polygons[PrimitiveIndex]
					if len(Mesh.materials) > 0:
						Material = Mesh.materials[Polygon.material_index]
					else:
						Material = None
					if not Material in Shaders:
						Shader2 = Shader(xml.etree.cElementTree.SubElement(MeshElement,"Shader"))
						if not Material is None:
							Shader2.Element.set("Name",Material.name)
						Shaders[Material] = Shader2
					else:
						Shader2 = Shaders[Material]
					I = 0
					while I < 3:
						LoopIndex = Polygon.loop_start+I
						Loop = Mesh.loops[LoopIndex]
						Normal = mathutils.Vector((Loop.normal.x,Loop.normal.z,Loop.normal.y))
						if not UVLayer is None:
							UVLayerData = UVLayer.data[LoopIndex]
							UV = mathutils.Vector(UVLayerData.uv)
						else:
							UV = mathutils.Vector((0,0))
						VertexIndex = Shader2.GetVertexIndex(Loop.vertex_index,Normal,UV)
						if VertexIndex is None:
							VertexIndex = len(Shader2.Vertices)
							Shader2.Vertices.append(Vertex(Loop.vertex_index,Normal,UV))
							VertexElement = xml.etree.cElementTree.SubElement(Shader2.Element,"Vertex")
							Vertex2 = Mesh.vertices[Loop.vertex_index]
							VertexElement.set("PositionX",str(Vertex2.co.x))
							VertexElement.set("PositionY",str(Vertex2.co.z))
							VertexElement.set("PositionZ",str(Vertex2.co.y))
							VertexElement.set("NormalX",str(Normal.x))
							VertexElement.set("NormalY",str(Normal.y))
							VertexElement.set("NormalZ",str(Normal.z))
							VertexElement.set("U",str(UV.x))
							VertexElement.set("V",str(UV.y))
							for Group in Vertex2.groups:
								MatrixElement = xml.etree.cElementTree.SubElement(VertexElement,"Matrix")
								MatrixElement.set("Name",Object.vertex_groups[Group.group].name)
								MatrixElement.set("Weight",str(Group.weight))
							VertexCount += 1
						LoopVertexIndices[LoopIndex] = VertexIndex
						I += 1
					Primitive = xml.etree.cElementTree.SubElement(Shader2.Element,"Primitive")
					Primitive.set("Vertex1",str(LoopVertexIndices[Polygon.loop_start+2]))
					Primitive.set("Vertex2",str(LoopVertexIndices[Polygon.loop_start+1]))
					Primitive.set("Vertex3",str(LoopVertexIndices[Polygon.loop_start]))
					PrimitiveIndex += 1
					PrimitiveCount += 1
				MeshElement.set("VertexCount",str(VertexCount))
				MeshElement.set("PrimitiveCount",str(PrimitiveCount))
	ElementTree = xml.etree.cElementTree.ElementTree(RootElement)
	ElementTree.write(filepath,encoding="utf-8",xml_declaration=True)
	return {"FINISHED"}