import bpy
import xml.etree.cElementTree
import os

def Load(Operator, Context, filepath):
	ElementTree = xml.etree.ElementTree.parse(filepath);
	RootElement = ElementTree.getroot()
	Scene = bpy.context.scene
	Joints2 = []
	Shaders = {}
	Textures = {}
	for Element in RootElement:
		if Element.tag == "Joint":
			MatrixName = Element.attrib["Name"]
			Joints2.append(MatrixName)
		elif Element.tag == "Shader":
			ShaderName = Element.attrib["Name"]
			Material = bpy.data.materials.new(ShaderName)
			if "TextureName" in Element.attrib:
				TextureName = Element.attrib["TextureName"]
				if TextureName in Textures:
					Texture = Textures[TextureName]
				else:
					try:
						Image = bpy.data.images.load(os.path.join(os.path.dirname(filepath),TextureName)+".png")
					except:
						Image = None
					Texture = bpy.data.textures.new(TextureName,type="IMAGE")
					Texture.image = Image
				Textures[TextureName] = Texture
				TextureSlot = Material.texture_slots.add()
				TextureSlot.texture = Texture
				TextureSlot.texture_coords = "UV"
				TextureSlot.use_map_color_diffuse = True
			Shaders[ShaderName] = Material
		elif Element.tag == "Mesh":
			MaterialIndex = 0
			UVs = []
			if "Name" in Element.attrib:
				Name = Element.attrib["Name"]
			else:
				Name = ""
			Joints = {}
			VertexCount = int(Element.attrib["VertexCount"])
			PrimitiveCount = int(Element.attrib["PrimitiveCount"])
			Mesh = bpy.data.meshes.new(Name)
			Mesh.vertices.add(VertexCount)
			Mesh.loops.add(PrimitiveCount*3)
			Mesh.polygons.add(PrimitiveCount)
			Mesh.create_normals_split()
			Mesh.uv_textures.new()
			UVLayer = Mesh.uv_layers[0]
			Object = bpy.data.objects.new(Name,Mesh)
			VertexIndex = 0
			PrimitiveIndex = 0
			for Joint in Joints2:
				MatrixIndex = len(Object.vertex_groups)
				Object.vertex_groups.new(Joint)
				Joints[Joint] = MatrixIndex
			for Element4 in Element:
				if Element4.tag == "Shader":
					ShaderFirstVertexIndex = VertexIndex
					ShaderName = Element4.attrib["Name"]
					Material = Shaders[ShaderName]
					Mesh.materials.append(Material)
					for Element2 in Element4:
						if Element2.tag == "Vertex":
							Mesh.vertices[VertexIndex].co = (float(Element2.attrib["PositionX"]),float(Element2.attrib["PositionZ"]),float(Element2.attrib["PositionY"]))
							Mesh.vertices[VertexIndex].normal = (float(Element2.attrib["NormalX"]),float(Element2.attrib["NormalZ"]),float(Element2.attrib["NormalY"]))
							if "U" in Element2.attrib:
								UVs.append((float(Element2.attrib["U"]),float(Element2.attrib["V"])))
							else:
								UVs.append(None)
							for Element3 in Element2:
								if Element3.tag == "Matrix":
									MatrixName = Element3.attrib["Name"]
									if not MatrixName in Joints:
										MatrixIndex = len(Object.vertex_groups)
										Object.vertex_groups.new(MatrixName)
										Joints[MatrixName] = MatrixIndex
									else:
										MatrixIndex = Joints[MatrixName]
									Object.vertex_groups[MatrixIndex].add([VertexIndex],float(Element3.attrib["Weight"]),"ADD")
							VertexIndex += 1
						elif Element2.tag == "Primitive":
							Loop1 = Mesh.loops[PrimitiveIndex*3+0]
							Loop2 = Mesh.loops[PrimitiveIndex*3+1]
							Loop3 = Mesh.loops[PrimitiveIndex*3+2]
							Loop1.vertex_index = ShaderFirstVertexIndex+int(Element2.attrib["Vertex3"]);
							Loop2.vertex_index = ShaderFirstVertexIndex+int(Element2.attrib["Vertex2"]);
							Loop3.vertex_index = ShaderFirstVertexIndex+int(Element2.attrib["Vertex1"]);
							Loop1.normal = Mesh.vertices[Loop1.vertex_index].normal
							Loop2.normal = Mesh.vertices[Loop2.vertex_index].normal
							Loop3.normal = Mesh.vertices[Loop3.vertex_index].normal
							Mesh.polygons[PrimitiveIndex].loop_start = PrimitiveIndex*3+0
							Mesh.polygons[PrimitiveIndex].loop_total = 3
							Mesh.polygons[PrimitiveIndex].material_index = MaterialIndex
							Mesh.polygons[PrimitiveIndex].use_smooth = True
							if not UVs[Loop1.vertex_index] is None:
								UVLayer.data[PrimitiveIndex*3+0].uv = UVs[Loop1.vertex_index]
							if not UVs[Loop2.vertex_index] is None:
								UVLayer.data[PrimitiveIndex*3+1].uv = UVs[Loop2.vertex_index]
							if not UVs[Loop3.vertex_index] is None:
								UVLayer.data[PrimitiveIndex*3+2].uv = UVs[Loop3.vertex_index]
							PrimitiveIndex += 1
					MaterialIndex += 1
			Mesh.validate(clean_customdata=False)
			Mesh.update(calc_edges=False)
			Normals = []
			LoopCount = len(Mesh.loops)
			I = 0
			while I < LoopCount:
				Normals.append(Mesh.loops[I].normal)
				I += 1
			Mesh.normals_split_custom_set(Normals)
			Mesh.use_auto_smooth = True
			Scene.objects.link(Object)
			Object.select = True
	return {"FINISHED"}